"use strict";
exports.__esModule = true;
exports.useComponentSize = void 0;
var tslib_1 = require("tslib");
var react_1 = require("react");
var getElementSize = function (el) {
    if (!el) {
        return { width: 0, height: 0 };
    }
    var _a = el.getBoundingClientRect(), width = _a.width, height = _a.height;
    return {
        width: Math.floor(width),
        height: Math.floor(height)
    };
};
function useComponentSize(ref) {
    var _a = tslib_1.__read(react_1.useState(getElementSize(ref && ref.current)), 2), componentSize = _a[0], setComponentSize = _a[1];
    var handleResize = react_1.useCallback(function () {
        if (ref.current) {
            setComponentSize(getElementSize(ref.current));
        }
    }, [ref]);
    react_1.useLayoutEffect(function () {
        if (!ref.current) {
            return;
        }
        handleResize();
        var resizeObserver = new ResizeObserver(handleResize);
        resizeObserver.observe(ref.current);
        return function () {
            resizeObserver.disconnect();
        };
    }, [ref, handleResize]);
    return componentSize;
}
exports.useComponentSize = useComponentSize;
