/// <reference types="react" />
import { PropsWithAsAttributes } from '../../../utils/types/PropsWithAsAttributes';
import { Props } from '../../Button/Button';
import { EventInterceptorHandler } from '../EventInterceptor';
export declare type ButtonProps<As extends keyof JSX.IntrinsicElements = 'button'> = PropsWithAsAttributes<Props, As>;
export declare const useButtonEventHandler: (props: ButtonProps, handler: EventInterceptorHandler) => {
    size?: "xs" | "s" | "m" | "l" | undefined;
    view?: "ghost" | "primary" | "secondary" | "clear" | undefined;
    width?: "default" | "full" | undefined;
    form?: "default" | "round" | "brick" | "defaultBrick" | "brickDefault" | "brickRound" | "roundBrick" | undefined;
    tabIndex?: number | undefined;
    disabled?: boolean | undefined;
    loading?: boolean | undefined;
    label?: string | number | undefined;
    onClick?: ((event: import("react").MouseEvent<Element, MouseEvent>) => void) | undefined;
    iconLeft?: import("react").FC<import("../../../utils/types/PropsWithHTMLAttributes").PropsWithHTMLAttributes<import("../../../icons/Icon/Icon").Props, HTMLSpanElement>> | undefined;
    iconRight?: import("react").FC<import("../../../utils/types/PropsWithHTMLAttributes").PropsWithHTMLAttributes<import("../../../icons/Icon/Icon").Props, HTMLSpanElement>> | undefined;
    onlyIcon?: boolean | undefined;
    iconSize?: "xs" | "s" | "m" | undefined;
    title?: string | undefined;
    children?: undefined;
    as?: "button" | undefined;
    css?: import("@emotion/core").InterpolationWithTheme<any>;
    defaultChecked?: boolean | undefined;
    defaultValue?: string | number | readonly string[] | undefined;
    suppressContentEditableWarning?: boolean | undefined;
    suppressHydrationWarning?: boolean | undefined;
    accessKey?: string | undefined;
    className?: string | undefined;
    contentEditable?: boolean | "true" | "false" | "inherit" | undefined;
    contextMenu?: string | undefined;
    dir?: string | undefined;
    draggable?: boolean | "true" | "false" | undefined;
    hidden?: boolean | undefined;
    id?: string | undefined;
    lang?: string | undefined;
    placeholder?: string | undefined;
    slot?: string | undefined;
    spellCheck?: boolean | "true" | "false" | undefined;
    style?: import("react").CSSProperties | undefined;
    translate?: "yes" | "no" | undefined;
    radioGroup?: string | undefined;
    role?: string | undefined;
    about?: string | undefined;
    datatype?: string | undefined;
    inlist?: any;
    prefix?: string | undefined;
    property?: string | undefined;
    resource?: string | undefined;
    typeof?: string | undefined;
    vocab?: string | undefined;
    autoCapitalize?: string | undefined;
    autoCorrect?: string | undefined;
    autoSave?: string | undefined;
    color?: string | undefined;
    itemProp?: string | undefined;
    itemScope?: boolean | undefined;
    itemType?: string | undefined;
    itemID?: string | undefined;
    itemRef?: string | undefined;
    results?: number | undefined;
    security?: string | undefined;
    unselectable?: "on" | "off" | undefined;
    inputMode?: "search" | "text" | "none" | "tel" | "url" | "email" | "numeric" | "decimal" | undefined;
    is?: string | undefined;
    'aria-activedescendant'?: string | undefined;
    'aria-atomic'?: boolean | "true" | "false" | undefined;
    'aria-autocomplete'?: "none" | "inline" | "list" | "both" | undefined;
    'aria-busy'?: boolean | "true" | "false" | undefined;
    'aria-checked'?: boolean | "true" | "false" | "mixed" | undefined;
    'aria-colcount'?: number | undefined;
    'aria-colindex'?: number | undefined;
    'aria-colspan'?: number | undefined;
    'aria-controls'?: string | undefined;
    'aria-current'?: boolean | "time" | "true" | "false" | "page" | "step" | "location" | "date" | undefined;
    'aria-describedby'?: string | undefined;
    'aria-details'?: string | undefined;
    'aria-disabled'?: boolean | "true" | "false" | undefined;
    'aria-dropeffect'?: "link" | "none" | "copy" | "execute" | "move" | "popup" | undefined;
    'aria-errormessage'?: string | undefined;
    'aria-expanded'?: boolean | "true" | "false" | undefined;
    'aria-flowto'?: string | undefined;
    'aria-grabbed'?: boolean | "true" | "false" | undefined;
    'aria-haspopup'?: boolean | "dialog" | "menu" | "true" | "false" | "listbox" | "tree" | "grid" | undefined;
    'aria-hidden'?: boolean | "true" | "false" | undefined;
    'aria-invalid'?: boolean | "true" | "false" | "grammar" | "spelling" | undefined;
    'aria-keyshortcuts'?: string | undefined;
    'aria-label'?: string | undefined;
    'aria-labelledby'?: string | undefined;
    'aria-level'?: number | undefined;
    'aria-live'?: "off" | "assertive" | "polite" | undefined;
    'aria-modal'?: boolean | "true" | "false" | undefined;
    'aria-multiline'?: boolean | "true" | "false" | undefined;
    'aria-multiselectable'?: boolean | "true" | "false" | undefined;
    'aria-orientation'?: "horizontal" | "vertical" | undefined;
    'aria-owns'?: string | undefined;
    'aria-placeholder'?: string | undefined;
    'aria-posinset'?: number | undefined;
    'aria-pressed'?: boolean | "true" | "false" | "mixed" | undefined;
    'aria-readonly'?: boolean | "true" | "false" | undefined;
    'aria-relevant'?: "text" | "additions" | "additions removals" | "additions text" | "all" | "removals" | "removals additions" | "removals text" | "text additions" | "text removals" | undefined;
    'aria-required'?: boolean | "true" | "false" | undefined;
    'aria-roledescription'?: string | undefined;
    'aria-rowcount'?: number | undefined;
    'aria-rowindex'?: number | undefined;
    'aria-rowspan'?: number | undefined;
    'aria-selected'?: boolean | "true" | "false" | undefined;
    'aria-setsize'?: number | undefined;
    'aria-sort'?: "none" | "ascending" | "descending" | "other" | undefined;
    'aria-valuemax'?: number | undefined;
    'aria-valuemin'?: number | undefined;
    'aria-valuenow'?: number | undefined;
    'aria-valuetext'?: string | undefined;
    dangerouslySetInnerHTML?: {
        __html: string;
    } | undefined;
    onCopy?: ((event: import("react").ClipboardEvent<HTMLButtonElement>) => void) | undefined;
    onCopyCapture?: ((event: import("react").ClipboardEvent<HTMLButtonElement>) => void) | undefined;
    onCut?: ((event: import("react").ClipboardEvent<HTMLButtonElement>) => void) | undefined;
    onCutCapture?: ((event: import("react").ClipboardEvent<HTMLButtonElement>) => void) | undefined;
    onPaste?: ((event: import("react").ClipboardEvent<HTMLButtonElement>) => void) | undefined;
    onPasteCapture?: ((event: import("react").ClipboardEvent<HTMLButtonElement>) => void) | undefined;
    onCompositionEnd?: ((event: import("react").CompositionEvent<HTMLButtonElement>) => void) | undefined;
    onCompositionEndCapture?: ((event: import("react").CompositionEvent<HTMLButtonElement>) => void) | undefined;
    onCompositionStart?: ((event: import("react").CompositionEvent<HTMLButtonElement>) => void) | undefined;
    onCompositionStartCapture?: ((event: import("react").CompositionEvent<HTMLButtonElement>) => void) | undefined;
    onCompositionUpdate?: ((event: import("react").CompositionEvent<HTMLButtonElement>) => void) | undefined;
    onCompositionUpdateCapture?: ((event: import("react").CompositionEvent<HTMLButtonElement>) => void) | undefined;
    onFocus?: ((event: import("react").FocusEvent<HTMLButtonElement>) => void) | undefined;
    onFocusCapture?: ((event: import("react").FocusEvent<HTMLButtonElement>) => void) | undefined;
    onBlur?: ((event: import("react").FocusEvent<HTMLButtonElement>) => void) | undefined;
    onBlurCapture?: ((event: import("react").FocusEvent<HTMLButtonElement>) => void) | undefined;
    onChange?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onChangeCapture?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onBeforeInput?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onBeforeInputCapture?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onInput?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onInputCapture?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onReset?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onResetCapture?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onSubmit?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onSubmitCapture?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onInvalid?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onInvalidCapture?: ((event: import("react").FormEvent<HTMLButtonElement>) => void) | undefined;
    onLoad?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onLoadCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onError?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onErrorCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onKeyDown?: ((event: import("react").KeyboardEvent<HTMLButtonElement>) => void) | undefined;
    onKeyDownCapture?: ((event: import("react").KeyboardEvent<HTMLButtonElement>) => void) | undefined;
    onKeyPress?: ((event: import("react").KeyboardEvent<HTMLButtonElement>) => void) | undefined;
    onKeyPressCapture?: ((event: import("react").KeyboardEvent<HTMLButtonElement>) => void) | undefined;
    onKeyUp?: ((event: import("react").KeyboardEvent<HTMLButtonElement>) => void) | undefined;
    onKeyUpCapture?: ((event: import("react").KeyboardEvent<HTMLButtonElement>) => void) | undefined;
    onAbort?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onAbortCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onCanPlay?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onCanPlayCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onCanPlayThrough?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onCanPlayThroughCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onDurationChange?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onDurationChangeCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onEmptied?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onEmptiedCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onEncrypted?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onEncryptedCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onEnded?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onEndedCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onLoadedData?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onLoadedDataCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onLoadedMetadata?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onLoadedMetadataCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onLoadStart?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onLoadStartCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onPause?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onPauseCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onPlay?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onPlayCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onPlaying?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onPlayingCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onProgress?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onProgressCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onRateChange?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onRateChangeCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onSeeked?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onSeekedCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onSeeking?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onSeekingCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onStalled?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onStalledCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onSuspend?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onSuspendCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onTimeUpdate?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onTimeUpdateCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onVolumeChange?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onVolumeChangeCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onWaiting?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onWaitingCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onAuxClick?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onAuxClickCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onClickCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onContextMenu?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onContextMenuCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onDoubleClick?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onDoubleClickCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onDrag?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragCapture?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragEnd?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragEndCapture?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragEnter?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragEnterCapture?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragExit?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragExitCapture?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragLeave?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragLeaveCapture?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragOver?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragOverCapture?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragStart?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDragStartCapture?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDrop?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onDropCapture?: ((event: import("react").DragEvent<HTMLButtonElement>) => void) | undefined;
    onMouseDown?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseDownCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseEnter?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseLeave?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseMove?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseMoveCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseOut?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseOutCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseOver?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseOverCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseUp?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onMouseUpCapture?: ((event: import("react").MouseEvent<HTMLButtonElement, MouseEvent>) => void) | undefined;
    onSelect?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onSelectCapture?: ((event: import("react").SyntheticEvent<HTMLButtonElement, Event>) => void) | undefined;
    onTouchCancel?: ((event: import("react").TouchEvent<HTMLButtonElement>) => void) | undefined;
    onTouchCancelCapture?: ((event: import("react").TouchEvent<HTMLButtonElement>) => void) | undefined;
    onTouchEnd?: ((event: import("react").TouchEvent<HTMLButtonElement>) => void) | undefined;
    onTouchEndCapture?: ((event: import("react").TouchEvent<HTMLButtonElement>) => void) | undefined;
    onTouchMove?: ((event: import("react").TouchEvent<HTMLButtonElement>) => void) | undefined;
    onTouchMoveCapture?: ((event: import("react").TouchEvent<HTMLButtonElement>) => void) | undefined;
    onTouchStart?: ((event: import("react").TouchEvent<HTMLButtonElement>) => void) | undefined;
    onTouchStartCapture?: ((event: import("react").TouchEvent<HTMLButtonElement>) => void) | undefined;
    onPointerDown?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerDownCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerMove?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerMoveCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerUp?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerUpCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerCancel?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerCancelCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerEnter?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerEnterCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerLeave?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerLeaveCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerOver?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerOverCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerOut?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onPointerOutCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onGotPointerCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onGotPointerCaptureCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onLostPointerCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onLostPointerCaptureCapture?: ((event: import("react").PointerEvent<HTMLButtonElement>) => void) | undefined;
    onScroll?: ((event: import("react").UIEvent<HTMLButtonElement, UIEvent>) => void) | undefined;
    onScrollCapture?: ((event: import("react").UIEvent<HTMLButtonElement, UIEvent>) => void) | undefined;
    onWheel?: ((event: import("react").WheelEvent<HTMLButtonElement>) => void) | undefined;
    onWheelCapture?: ((event: import("react").WheelEvent<HTMLButtonElement>) => void) | undefined;
    onAnimationStart?: ((event: import("react").AnimationEvent<HTMLButtonElement>) => void) | undefined;
    onAnimationStartCapture?: ((event: import("react").AnimationEvent<HTMLButtonElement>) => void) | undefined;
    onAnimationEnd?: ((event: import("react").AnimationEvent<HTMLButtonElement>) => void) | undefined;
    onAnimationEndCapture?: ((event: import("react").AnimationEvent<HTMLButtonElement>) => void) | undefined;
    onAnimationIteration?: ((event: import("react").AnimationEvent<HTMLButtonElement>) => void) | undefined;
    onAnimationIterationCapture?: ((event: import("react").AnimationEvent<HTMLButtonElement>) => void) | undefined;
    onTransitionEnd?: ((event: import("react").TransitionEvent<HTMLButtonElement>) => void) | undefined;
    onTransitionEndCapture?: ((event: import("react").TransitionEvent<HTMLButtonElement>) => void) | undefined;
    ref?: string | ((instance: HTMLButtonElement | null) => void) | import("react").RefObject<HTMLButtonElement> | null | undefined;
    key?: string | number | null | undefined;
    name?: string | undefined;
    type?: "button" | "submit" | "reset" | undefined;
    autoFocus?: boolean | undefined;
    formAction?: string | undefined;
    formEncType?: string | undefined;
    formMethod?: string | undefined;
    formNoValidate?: boolean | undefined;
    formTarget?: string | undefined;
    value?: string | number | readonly string[] | undefined;
};
