import { __assign, __read, __rest, __spread } from "tslib";
import React, { useRef, useState } from 'react';
import { useSelect } from '../../hooks/useSelect/useSelect';
import { IconClose } from '../../icons/IconClose/IconClose';
import { IconSelect } from '../../icons/IconSelect/IconSelect';
import { cnMixFocus } from '../../mixs/MixFocus/MixFocus';
import { scrollIntoView } from '../../utils/scrollIntoView';
import { cnSelect } from '../SelectComponents/cnSelect';
import { getSelectDropdownForm } from '../SelectComponents/helpers';
import { SelectContainer } from '../SelectComponents/SelectContainer/SelectContainer';
import { SelectDropdown } from '../SelectComponents/SelectDropdown/SelectDropdown';
import { SelectValueTag } from '../SelectComponents/SelectValueTag/SelectValueTag';
import { DefaultPropForm, DefaultPropView } from '../SelectComponents/types';
export var multiComboboxPropSize = ['m', 's', 'l'];
export var multiComboboxPropSizeDefault = multiComboboxPropSize[0];
export var MultiCombobox = function (props) {
    var defaultOptionsRef = useRef(null);
    var placeholder = props.placeholder, onBlur = props.onBlur, onFocus = props.onFocus, options = props.options, onChange = props.onChange, value = props.value, getOptionLabel = props.getOptionLabel, disabled = props.disabled, ariaLabel = props.ariaLabel, id = props.id, _a = props.form, form = _a === void 0 ? DefaultPropForm : _a, _b = props.view, view = _b === void 0 ? DefaultPropView : _b, _c = props.size, size = _c === void 0 ? multiComboboxPropSizeDefault : _c, onCreate = props.onCreate, getGroupOptions = props.getGroupOptions, _d = props.labelForCreate, labelForCreate = _d === void 0 ? 'Добавить' : _d, _e = props.labelForNotFound, labelForNotFound = _e === void 0 ? 'Не найдено' : _e, dropdownClassName = props.dropdownClassName, _f = props.dropdownRef, dropdownRef = _f === void 0 ? defaultOptionsRef : _f, name = props.name, restProps = __rest(props, ["placeholder", "onBlur", "onFocus", "options", "onChange", "value", "getOptionLabel", "disabled", "ariaLabel", "id", "form", "view", "size", "onCreate", "getGroupOptions", "labelForCreate", "labelForNotFound", "dropdownClassName", "dropdownRef", "name"]);
    var _g = __read(useState(false), 2), isFocused = _g[0], setIsFocused = _g[1];
    var _h = __read(useState({
        value: ''
    }), 2), inputData = _h[0], setInputData = _h[1];
    var toggleRef = useRef(null);
    var controlInnerRef = useRef(null);
    var helperInputFakeElement = useRef(null);
    var handlerChangeValue = function (values) {
        var _a;
        if (typeof onChange === 'function' && values) {
            onChange(values.length === 0 ? null : values);
        }
        setInputData({ value: (_a = toggleRef.current) === null || _a === void 0 ? void 0 : _a.value });
    };
    var controlRef = useRef(null);
    var arrValue = typeof value !== 'undefined' && value !== null ? __spread(value) : null;
    var hasGroup = typeof getGroupOptions === 'function';
    var scrollToIndex = function (index) {
        if (!dropdownRef.current) {
            return;
        }
        var elements = dropdownRef.current.querySelectorAll('div[role=option]');
        scrollIntoView(elements[index], dropdownRef.current);
    };
    var onSelectOption = function () {
        setInputData({ value: '' });
    };
    var _j = useSelect({
        options: options,
        value: arrValue,
        onChange: handlerChangeValue,
        optionsRef: dropdownRef,
        controlRef: controlRef,
        scrollToIndex: scrollToIndex,
        disabled: disabled,
        getOptionLabel: getOptionLabel,
        onCreate: onCreate,
        getGroupOptions: getGroupOptions,
        multi: true,
        onSelectOption: onSelectOption
    }), visibleOptions = _j.visibleOptions, highlightedIndex = _j.highlightedIndex, getToggleProps = _j.getToggleProps, getOptionProps = _j.getOptionProps, isOpen = _j.isOpen, setOpen = _j.setOpen;
    var handleInputFocus = function (e) {
        if (!disabled) {
            if (!isFocused) {
                setIsFocused(true);
            }
            if (typeof onFocus === 'function') {
                onFocus(e);
            }
        }
    };
    var handleInputBlur = function (e) {
        var _a;
        if (isOpen) {
            (_a = toggleRef.current) === null || _a === void 0 ? void 0 : _a.focus();
            return;
        }
        if (isFocused) {
            setIsFocused(false);
            if (typeof onBlur === 'function') {
                onBlur(e);
            }
        }
    };
    var handleToggleDropdown = function () {
        var _a;
        if (isOpen) {
            setOpen(false);
            setIsFocused(false);
        }
        else {
            setOpen(true);
            setIsFocused(true);
            (_a = toggleRef.current) === null || _a === void 0 ? void 0 : _a.focus();
        }
    };
    var handleClear = function () {
        setInputData({ value: '' });
        typeof onChange === 'function' && onChange(null);
        setIsFocused(false);
    };
    var handleClearButtonFocus = function () {
        setIsFocused(true);
    };
    var handleClearButtonBlur = function () {
        setIsFocused(false);
    };
    var handleInputChange = function () {
        var _a, _b;
        if (!isOpen) {
            setOpen(true);
        }
        var inputValue = (_b = (_a = toggleRef.current) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : '';
        setInputData({ value: inputValue });
    };
    var handleControlClick = function () {
        var _a;
        (_a = toggleRef.current) === null || _a === void 0 ? void 0 : _a.focus();
        if (!isFocused) {
            setOpen(!isOpen);
        }
    };
    var showPlaceholder = (!(arrValue === null || arrValue === void 0 ? void 0 : arrValue.length) && inputData.value === '') || (arrValue === null && inputData.value === '');
    var handleCreate = function () {
        var _a;
        if (typeof onCreate === 'function') {
            var newValue = (_a = toggleRef.current) === null || _a === void 0 ? void 0 : _a.value.trim();
            newValue && onCreate(newValue);
        }
    };
    var handleRemoveValue = function (option) {
        var newVal = arrValue === null || arrValue === void 0 ? void 0 : arrValue.filter(function (arrVal) { return getOptionLabel(arrVal) !== getOptionLabel(option); });
        handlerChangeValue(newVal);
    };
    var getInputStyle = function () {
        if (!controlInnerRef.current || !helperInputFakeElement.current) {
            return {
                width: 0
            };
        }
        var fakeElWidth = helperInputFakeElement.current.offsetWidth + 20;
        var maxWidth = controlInnerRef.current ? controlInnerRef.current.offsetWidth - 15 : 0;
        var width = fakeElWidth > maxWidth ? maxWidth : fakeElWidth;
        return {
            width: width
        };
    };
    var inputStyle = React.useMemo(function () { return getInputStyle(); }, [inputData.value, arrValue]);
    return (React.createElement(SelectContainer, __assign({ focused: isFocused, disabled: disabled, size: size, view: view, form: form, multi: true }, restProps),
        React.createElement("div", { className: cnSelect('Control', { hasInput: true }), ref: controlRef, "aria-expanded": isOpen, "aria-haspopup": "listbox", id: id },
            React.createElement("div", { className: cnSelect('ControlInner'), onClick: handleControlClick, onKeyDown: handleControlClick, role: "button", ref: controlInnerRef },
                React.createElement("div", { className: cnSelect('ControlValueContainer') },
                    React.createElement("div", { className: cnSelect('ControlValue') }, arrValue === null || arrValue === void 0 ? void 0 :
                        arrValue.map(function (option) {
                            var label = getOptionLabel(option);
                            var handleRemove = function (e) {
                                e.stopPropagation();
                                handleRemoveValue(option);
                            };
                            return (React.createElement(SelectValueTag, { label: label, key: label, size: size, disabled: Boolean(disabled), handleRemove: handleRemove }));
                        }),
                        React.createElement("input", __assign({}, getToggleProps({ onChange: handleInputChange }), { type: "text", name: name, onFocus: handleInputFocus, onBlur: handleInputBlur, "aria-label": ariaLabel, ref: toggleRef, value: inputData.value, className: cnSelect('Input', { size: size }), style: inputStyle })),
                        React.createElement("div", { className: cnSelect('Placeholder', { isHidden: !showPlaceholder }) }, placeholder)))),
            React.createElement("div", { className: cnSelect('Indicators') },
                arrValue && arrValue.length !== 0 && (React.createElement("button", { type: "button", onClick: handleClear, className: cnSelect('ClearIndicator', [cnMixFocus()]), onFocus: handleClearButtonFocus, onBlur: handleClearButtonBlur },
                    React.createElement(IconClose, { size: "xs", className: cnSelect('ClearIndicatorIcon') }))),
                React.createElement("div", { className: cnSelect('Delimiter') }),
                React.createElement("button", { type: "button", className: cnSelect('IndicatorsDropdown'), tabIndex: -1, onClick: handleToggleDropdown },
                    React.createElement(IconSelect, { size: "xs", className: cnSelect('DropdownIndicatorIcon') })))),
        React.createElement(SelectDropdown, { isOpen: isOpen, size: size, controlRef: controlRef, visibleOptions: visibleOptions, highlightedIndex: highlightedIndex, getOptionProps: getOptionProps, onCreate: handleCreate, dropdownRef: dropdownRef, inputValue: inputData.value, id: id, hasGroup: hasGroup, selectedValues: arrValue, labelForCreate: labelForCreate, labelForNotFound: labelForNotFound, multi: true, getOptionLabel: getOptionLabel, form: getSelectDropdownForm(form), className: dropdownClassName }),
        React.createElement("div", { className: cnSelect('HelperInputFakeElement'), ref: helperInputFakeElement }, inputData.value)));
};
