import { __assign, __rest, __values } from "tslib";
import './Tabs.css';
import React, { createRef, useEffect, useMemo, useRef } from 'react';
import { useChoiceGroup } from '../../hooks/useChoiceGroup/useChoiceGroup';
import { useForkRef } from '../../hooks/useForkRef/useForkRef';
import { cn } from '../../utils/bem';
import { getSizeByMap } from '../../utils/getSizeByMap';
import { cnTabsTab, TabsTab } from './Tab/TabsTab';
export var tabsSizes = ['m', 's'];
export var tabsDefaultSize = tabsSizes[0];
export var tabsViews = ['bordered', 'clear'];
export var tabsDefaultView = tabsViews[0];
export var cnTabs = cn('Tabs');
var sizeMap = {
    s: 'xs',
    m: 's'
};
function setStyleForLine(lineRef, tabsWidth, tabWidth, tabRatio, tabOffsetLeft) {
    if (lineRef.current) {
        var lineStyle = lineRef.current.style;
        lineStyle.setProperty('--tabsWidth', tabsWidth + "px");
        lineStyle.setProperty('--tabWidth', tabWidth + "px");
        lineStyle.setProperty('--tabRatio', "" + tabRatio);
        lineStyle.setProperty('--tabOffsetLeft', tabOffsetLeft + "px");
    }
}
function renderItemDefault(props) {
    var ref = props.ref, onChange = props.onChange, otherProps = __rest(props, ["ref", "onChange"]);
    return (React.createElement(TabsTab, __assign({}, otherProps, { ref: ref, onChange: onChange })));
}
export var Tabs = React.forwardRef(function (props, ref) {
    var _a = props.size, size = _a === void 0 ? tabsDefaultSize : _a, className = props.className, items = props.items, _b = props.view, view = _b === void 0 ? tabsDefaultView : _b, value = props.value, onlyIcon = props.onlyIcon, getIcon = props.getIcon, getLabel = props.getLabel, onChange = props.onChange, iconSizeProp = props.iconSize, _c = props.renderItem, renderItem = _c === void 0 ? renderItemDefault : _c, otherProps = __rest(props, ["size", "className", "items", "view", "value", "onlyIcon", "getIcon", "getLabel", "onChange", "iconSize", "renderItem"]);
    var _d = useChoiceGroup({
        value: value || null,
        getKey: getLabel,
        callBack: onChange,
        multiple: false
    }), getOnChange = _d.getOnChange, getChecked = _d.getChecked;
    var constructItemRefs = function () {
        var e_1, _a;
        var refs = {};
        try {
            for (var items_1 = __values(items), items_1_1 = items_1.next(); !items_1_1.done; items_1_1 = items_1.next()) {
                var item = items_1_1.value;
                refs[getLabel(item)] = createRef();
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (items_1_1 && !items_1_1.done && (_a = items_1["return"])) _a.call(items_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return refs;
    };
    var buttonRefs = useMemo(constructItemRefs, [items, getLabel]);
    var rootRef = useRef(null);
    var lineRef = useRef(null);
    var updateLine = function () {
        if (rootRef.current && lineRef.current && buttonRefs) {
            var rootWidth = rootRef.current.offsetWidth;
            if (value) {
                var activeItemRef = buttonRefs[getLabel(value)];
                if (activeItemRef && activeItemRef.current) {
                    var itemWidth = activeItemRef.current.offsetWidth;
                    var itemOffsetLeft = activeItemRef.current.offsetLeft;
                    setStyleForLine(lineRef, rootWidth, itemWidth, itemWidth / rootWidth, itemOffsetLeft);
                }
            }
            else {
                setStyleForLine(lineRef, rootWidth, 1, 0.00001, 1);
            }
        }
    };
    useEffect(function () { return updateLine(); });
    var withOutValue = !value;
    var iconSize = getSizeByMap(sizeMap, size, iconSizeProp);
    return (React.createElement("div", __assign({ className: cnTabs({ size: size, view: view }, [className]), ref: useForkRef([ref, rootRef]) }, otherProps),
        React.createElement("div", { className: cnTabs('List') }, items.map(function (item) {
            return renderItem({
                item: item,
                ref: buttonRefs[getLabel(item)],
                key: getLabel(item),
                onChange: getOnChange(item),
                checked: getChecked(item),
                label: getLabel(item).toString(),
                icon: getIcon && getIcon(item),
                iconSize: iconSize,
                onlyIcon: onlyIcon,
                className: cnTabs('Tab')
            });
        })),
        React.createElement("div", { className: cnTabs('WrapperRunningLine') },
            React.createElement("div", { className: cnTabs('RunningLine', { withOutValue: withOutValue }), ref: lineRef }))));
});
export { TabsTab, cnTabsTab };
